@@ -9,52 +9,105 @@ use English;
 use Exporter qw(import);
 use List::Util qw(min max);
 use Music::Harmonica::TabsCreator::NoteToToneConverter;
+use Music::Harmonica::TabsCreator::TabParser;
 use Readonly;
 use Scalar::Util qw(looks_like_number);
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
-our @EXPORT_OK = qw(tune_to_tab get_harmonica_details tune_to_tab_rendered);
+our @EXPORT_OK = qw(tune_to_tab get_tuning_details tune_to_tab_rendered
+    transpose_tab transpose_tab_rendered list_tunings);
 
 # Options to add:
 # - print B as H (international convention), but probably not Bb which stays Bb.
 
 Readonly my $TONES_PER_SCALE => 12;
 
-Readonly my %tunings => (
+# TODO: don’t use separate tunings with and without bending, add the bending
+# information in the main tuning and generate the missing data (e.g. how many
+# max half-step can be bent on a given hole, plus how many is the user
+# accepting).
+Readonly my %ALL_TUNINGS => (
   # Written in the key of C to match the default key used in the note_to_tone
   # function.
-  richter_no_bend => {
-    tags => [qw(diatonic 10-holes)],
-    name => 'Richter-tuned no bend',
-    # We arbitrarily keep only +3 and never use -2.
-    # We might need to change that if we wanted to support chords.
-    tab => [qw(  1  -1 2   3 -3  4 -4 5  -5 6  -6 7  -7 8  -8 9  -9 10 -10)],
-    notes => [qw(C4 D4 E4 G4 B4 C5 D5 E5 F5 G5 A5 C6 B5 E6 D6 G6 F6 C7 A6)],
-  },
+  # Note that when we have the same note appear multiple time (like -2 and 3 in
+  # Richter scale) we always use only the last appearance one when rendering a
+  # tab (but other appearances are still used when reading a tab).
   richter => {
     tags => [qw(diatonic 10-holes)],
-    name => 'Richter-tuned with bending',
+    name => 'Richter',
+    # Although we We arbitrarily keep only +3 and never use -2.
+    # We might need to change that if we wanted to support chords.
+    tab => [qw(  1  -1 2 -2  3  -3  4 -4 5  -5 6  -6 7  -7 8  -8 9  -9 10 -10)],
+    notes => [qw(C4 D4 E4 G4 G4 B4 C5 D5 E5 F5 G5 A5 C6 B5 E6 D6 G6 F6 C7 A6)],
+    bends => [qw(0  1  0  2  0  3  0  1  0  0  0  1  0  0  1  0  1  0  2  0)],
+  },
+  harmonic_minor => {
+    tags => [qw(diatonic 10-holes)],
+    name => 'Harmonic minor',
     # We arbitrarily keep only +3 and never use -2.
-    tab => [
-      qw(  1  -1 -1' 2  -2' -2" 3  -3 -3' -3" -3"' 4  -4 -4' 5  -5 6  -6 -6' 7  -7 8  8'  -8 9  9'  -9 10 10' 10" -10)
-    ],
-    notes => [
-      qw(C4 D4 Db4 E4 Gb4 F4  G4 B4 Bb4 A4  Ab4  C5 D5 Db5 E5 F5 G5 A5 Ab5 C6 B5 E6 Eb6 D6 G6 Gb6 F6 C7 B6  Bb6 A6)
-    ],
+    # We might need to change that if we wanted to support chords.
+    tab => [qw(1  -1 2   -2 3  -3 4 -4  5   -5 6  -6  7  -7 8   -8 9  -9 10 -10)],
+    notes => [qw(C4 D4 Eb4 G4 G4 B4 C5 D5 Eb5 F5 G5 Ab5 C6 B5 Eb6 D6 G6 F6 C7 Ab6)],
+    bends => [qw(0  1  0   3  0  3  0  1  0   1  0  0   0  0  0   0  1  0  3  0)],
   },
 );
 
 # We can’t use qw() because of the # that triggers a warning.
 Readonly my @keys_offset => split / /, q(C Db D Eb E F F# G Ab A Bb B);
 
-sub tune_to_tab ($sheet) {
+Readonly my $MAX_BENDS => 6;  # Probably higher than any realistic value.
+
+sub tune_to_tab ($sheet, %options) {
   my $note_converter = Music::Harmonica::TabsCreator::NoteToToneConverter->new();
   my @tones = $note_converter->convert($sheet);
+  my $tunings = generate_tunings($options{max_bends} // 0);
+  return find_matching_tuning(\@tones, $tunings);
+}
 
+sub transpose_tab ($tab, $tuning_id, $key, %options) {
+  # TODO: error handling for missing tuning
+  die "Unknown tuning: $tuning_id\n" unless exists $ALL_TUNINGS{$tuning_id};
+  # For the input, we accept any level of bending.
+  my $tuning = generate_tunings($MAX_BENDS)->{$tuning_id};
+  my $note_converter = Music::Harmonica::TabsCreator::NoteToToneConverter->new();
+  my %tab_to_tones = map { $tuning->{tab}[$_] => $note_converter->convert($tuning->{notes}[$_]) }
+      0 .. $#{$tuning->{tab}};
+  my $parser = Music::Harmonica::TabsCreator::TabParser->new(\%tab_to_tones);
+  my @tones = $parser->parse($tab);
+  my @key_tone = eval { $note_converter->convert($key) };
+  return "Invalid key: $key" if $@ || @key_tone != 1;
+  my $key_tone = $key_tone[0];
+  @tones = map { looks_like_number($_) ? $_ + $key_tone : $_ } @tones;
+  my $tunings = generate_tunings($options{max_bends} // 0);
+  return find_matching_tuning(\@tones, $tunings);
+}
+
+# We take the global %ALL_TUNINGS and generate a %tunings hash with the same
+# keys but where the values only have the notes and tab entries. But we have
+# added the notes corresponding to the allowed bends.
+# TODO: add an option to filter down the tunings that we are keeping.
+sub generate_tunings ($max_bends) {
+  my %out;
+  while (my ($k, $v) = each %ALL_TUNINGS) {
+    for my $i (0 .. $#{$v->{notes}}) {
+      my $note = $v->{notes}[$i];
+      my $tab = $v->{tab}[$i];
+      for my $b (0 .. min($max_bends, $v->{bends}[$i])) {
+        my $bent = (substr $note, 0, 1).('b' x $b).(substr $note, 1);
+        push @{$out{$k}{notes}}, $bent;
+        # TODO: this won’t work once we have chromatic harmonicas
+        push @{$out{$k}{tab}}, $tab.('"' x ($b / 2)).("'" x ($b % 2));
+      }
+    }
+  }
+  return \%out;
+}
+
+sub find_matching_tuning ($tones, $tunings) {
   my %all_matches;
-  while (my ($k, $v) = each %tunings) {
-    my @matches = match_notes_to_tuning(\@tones, $v);
+  while (my ($k, $v) = each %{$tunings}) {
+    my @matches = match_notes_to_tuning($tones, $v);
     for my $m (@matches) {
       push @{$all_matches{$k}{$m->[1]}}, $m->[0];
     }
@@ -62,12 +115,19 @@ sub tune_to_tab ($sheet) {
   return %all_matches;
 }
 
-sub tune_to_tab_rendered ($sheet) {
-  my %tabs = eval { tune_to_tab($sheet) };
-  if ($@) {
-    return $@;
-  }
+sub tune_to_tab_rendered ($sheet, %options) {
+  my %tabs = eval { tune_to_tab($sheet, %options) };
+  return $@ if $@;
+  return render_tabs(%tabs);
+}
 
+sub transpose_tab_rendered ($tab, $tuning, $key, %options) {
+  my %tabs = eval { transpose_tab($tab, $tuning, $key, %options) };
+  return $@ if $@;
+  return render_tabs(%tabs);
+}
+
+sub render_tabs (%tabs) {
   if (!%tabs) {
     return 'No tabs found';
   }
@@ -75,8 +135,8 @@ sub tune_to_tab_rendered ($sheet) {
   my $out;
 
   for my $type (sort keys %tabs) {
-    my %details = get_harmonica_details($type);
-    $out .= sprintf "For %s %s harmonicas:\n", join(' ', @{$details{tags}}), $details{name};
+    my %details = get_tuning_details($type);
+    $out .= sprintf "For %s %s tuning harmonicas:\n", join(' ', @{$details{tags}}), $details{name};
     for my $key (sort keys %{$tabs{$type}}) {
       $out .= "  In the key of ${key}:\n";
       for my $tab (@{$tabs{$type}{$key}}) {
@@ -88,8 +148,13 @@ sub tune_to_tab_rendered ($sheet) {
   return $out;
 }
 
-sub get_harmonica_details ($key) {
-  return %{$tunings{$key}}{qw(name tags)};
+sub get_tuning_details ($key) {
+  return %{$ALL_TUNINGS{$key}}{qw(name tags)};
+}
+
+sub list_tunings () {
+  return map { {id => $_, name => $ALL_TUNINGS{$_}{name}, tags => $ALL_TUNINGS{$_}{tags}} }
+      sort keys %ALL_TUNINGS;
 }
 
 # Given all the tones (with C0 = 0) of a melody and the data of a given
